@@ -15,8 +15,10 @@ from typing import Callable, Dict, Iterable, List, Optional, Protocol, Set
 import futureproof
 from eth_bloom import BloomFilter
 from futureproof import ThreadPoolExecutor
+from hexbytes import HexBytes
 from web3 import Web3
 from web3.contract.contract import ContractEvent
+from web3.datastructures import AttributeDict
 
 from eth_defi.event_reader.filter import Filter
 from eth_defi.event_reader.logresult import LogContext, LogResult
@@ -92,6 +94,10 @@ class Web3EventReader(Protocol):
 
     For concrete implementation see
 
+    - :py:func:`read_events`
+
+    - :py:func:`read_events_concurrent`
+
     - :py:func:`extract_events`
 
     - :py:func:`extract_events_concurrent`
@@ -175,6 +181,8 @@ def extract_events(
 ) -> Iterable[LogResult]:
     """Perform eth_getLogs call over a block range.
 
+    You should use :py:func:`read_events` unless you know the block range is something your node can handle.
+
     :param start_block:
         First block to process (inclusive)
 
@@ -185,7 +193,12 @@ def extract_events(
         Internal filter used to match logs
 
     :param extract_timestamps:
-        Method to get the block timestamps
+        Method to get the block timestamps.
+
+        This might need to use expensive`eth_getBlockByNumber` JSON-RPC API call.
+        It will seriously slow down event reading.
+        Set `extract_timestamps` to `None` to not get timestamps, but fast event lookups.
+
 
     :param context:
         Passed to the all generated logs
@@ -235,7 +248,13 @@ def extract_events(
             block_number = convert_jsonrpc_value_to_int(log["blockNumber"])
             # Retrofit our information to the dict
             event_signature = log["topics"][0]
+
+            if isinstance(log, AttributeDict):
+                # The following code is not going to work, because AttributeDict magic
+                raise RuntimeError("AttributeDict middleware detected. Please remove it with web3.middleware_onion.clear() before attempting to read events")
+
             log["context"] = context
+
             log["event"] = filter.topics[event_signature]
 
             # Can be hex string or integer (EthereumTester)
@@ -276,6 +295,8 @@ def extract_events_concurrent(
 ) -> List[LogResult]:
     """Concurrency happy event extractor.
 
+    You should use :py:func:`read_events_concurrent` unless you know the block range is something your node can handle.
+
     Called by the thread pool - you probably do not want to call this directly.
 
     Assumes the web3 connection is preset when the concurrent worker has been created,
@@ -394,7 +415,11 @@ def read_events(
         Last block to process (inclusive)
 
     :param extract_timestamps:
-        Override for different block timestamp extraction methods
+        Override for different block timestamp extraction methods.
+
+        This might need to use expensive`eth_getBlockByNumber` JSON-RPC API call.
+        It will seriously slow down event reading.
+        Set `extract_timestamps` to `None` to not get timestamps, but fast event lookups.
 
     :param chunk_size:
         How many blocks to scan in one eth_getLogs call
@@ -584,7 +609,6 @@ def read_events_concurrent(
 
     for block_num in range(start_block, end_block + 1, chunk_size):
         last_of_chunk = min(end_block, block_num + chunk_size - 1)
-
         task_list[block_num] = (
             block_num,
             last_of_chunk,
@@ -646,14 +670,12 @@ def read_events_concurrent(
                 # Pass through the logs and do timestamp resolution for them
                 #
                 for log in log_results:
-
                     # Check that this event is not from an alternative chain tip
                     if reorg_mon:
                         timestamp = reorg_mon.check_block_reorg(
                             convert_jsonrpc_value_to_int(log["blockNumber"]),
                             log["blockHash"],
                         )
-
                         last_timestamp = log["timestamp"] = timestamp
                     else:
                         # Assume extracted with extract_timestamps_json_rpc
